/*
 * Example for GSM/GNSS 2 click with SIM868 module
 *
 * Date         Jan 2017
 * Author       Stefan Popovic
 * Copyright    GNU Public License v2
 *
 * Test configuration FT90x :
 *  MCU           :        FT900
 *  Dev. Board    :        EasyFT90x
 *  SW            :        mikroC PRO for FT90x v2.0.0
 *
 *  NOTE          :
 * Include example.pld file in your project
 * Turn SW2.2 and SW2.3 in right position ( USB UART )
 * Make sure to type receiver's number and to rename microSD (D: as default)
 * Module first sends initial message to the user and then logs an events such as
 * calls or messages on microSD card and reads current GPS location,
 * all in an infinite loop.
 *
 ******************************************************************************/
#include "at_engine.h"

/*
 * GSM / GNSS 2 reset pin
 */
sbit GSM_GNSS_2_RST at GPIO_PIN1_bit;

/*
 * Power OFF String
 */
const char powerdown[] = "\r\nNORMAL POWER DOWN";

/*
 * Ready Flag
 */
bool ready_f = false;

/*
 * SIM Action Flag
 */
bool action_f = false;

/*
 * Message Ready Flag
 */
bool msg_ready_f = false;

/*
 * Message Delivered Flag
 */
bool msg_sent = false;

/*
 * microSD end Flag
 */
bool sd_f = false;

/*
 * Message event registered
 */
bool isMessage = false;

/*
 * Call event registered
 */
bool isCall = false;

/*
 * Response buffer
 */
uint8_t buffer[ 1024 ] = { 0 };

/*
 * Receiver ID ( Enter your phone number here )
 */
char receiver_ID[ 20 ] = "\"xxxxxxxxxxx\"";

/*
 * Number of calls
 */
 uint8_t call_counter;
 
/*
 * Number of messages
 */
 uint8_t msg_counter;
 
/* 
 * Textual representation of numbers 
 */
 char txt_call[ 5 ] = { 0 };
 char txt_msg[ 5 ] = { 0 };

/*
 * System Initialization
 *    - GPIO Initialization
 *        + GSM / GNSS 2 RST pin
 *    - Peripherals Initialization
 *        + UART1 for data dump
 *        + UART2 for SIM868
 *    - Interrupt Initialization
 *        + Timer
 *        + UART
 */
static void system_init( void )
{
    // Setting CS pin as output
    GPIO_Digital_Output ( &GPIO_PORT_00_07, _GPIO_PINMASK_1 );

    // Initialize hardware UART1 module at 115200 bps with 8 data, 
    // no parity and 1 stop bit 
    UART1_Init_Advanced( 115200, 
                         _UART_DATA_BITS_8, 
                         _UART_PARITY_NONE, 
                         _UART_STOP_BITS_1 );
    
    Delay_ms( 100 );
    
    // Initialize hardware UART2 module at 115200 bps with 8 data,
    // no parity and 1 stop bit 
    UART2_Init_Advanced( 115200, 
                         _UART_DATA_BITS_8, 
                         _UART_PARITY_NONE, 
                         _UART_STOP_BITS_1 );
    
    Delay_ms( 100 );

    // Timer Int Init
    TIMER_CONTROL_0 = 2;
    TIMER_SELECT = 0;
    TIMER_PRESC_LS = 80;
    TIMER_PRESC_MS = 195;
    TIMER_WRITE_LS = 1;
    TIMER_WRITE_MS = 0;
    TIMER_CONTROL_3 = 0;
    TIMER_CONTROL_4 |= 17;
    TIMER_CONTROL_2 |= 16;
    TIMER_INT |= 2;
    TIMER_CONTROL_1 |= 1;
    
    // UART2 Int Init
    UART2_FCR |= ( 1 << FIFO_EN ) | ( 1 << RCVR_RST );
    UART2_INT_ENABLE |= ( 1 << RX_AVL_EN );
    IRQ_CTRL &= ~( 1 << GLOBAL_INTERRUPT_MASK );

    msg_counter = 0;
    call_counter = 0;

    LOG( "\r\n**********************************************************\r\n" );
    LOG( "System Initialized.");
    LOG( "\r\n**********************************************************\r\n" );
}

/*
 * Power ON routine for GSM / GNSS 2
 */
static void gsm_gnss_2_power_on( void )
{   
    // Apply reset procedure
    at_cmd_single( "AT+CPOWD=1" );
    GSM_GNSS_2_RST = 1;
    Delay_ms( 1000 );
    GSM_GNSS_2_RST = 0;
    Delay_ms( 1500 );
    GSM_GNSS_2_RST = 1;
    
    LOG( "\r\n**********************************************************\r\n" );
    LOG( "Waiting For Device To Be Ready." );
    LOG( "\r\n**********************************************************\r\n" );

    Delay_ms( 20000 );                           // Waiting for GPS to stabilize
    
    LOG( "\r\n**********************************************************\r\n" );
    LOG( "Ready For Work." );
    LOG( "\r\n**********************************************************\r\n" );
}

void gsm_gnss_2_init( void )
{   
    LOG( "\r\n**********************************************************\r\n" );
    LOG( "GSM / GNSS 2 Initialization Started." );
    LOG( "\r\n**********************************************************\r\n" );
    
    // Enable Report of Mobile Equipment Error
    at_cmd_single( "AT+CMEE=2" );

    // Select Text SMS Message Format
    at_cmd_single( "AT+CMGF=1" );
     
    // Get SIM card group identifier
    at_cmd_single( "AT+CGID" );
    
    // List Available TE Character Set
    at_cmd_single( "AT+CSCS=?" );

    // Set TE Character Set as GSM 
    at_cmd_single( "AT+CSCS=\"GSM\"" );
    
    // Set New SMS Message Indications
    at_cmd_single( "AT+CNMI=2,2,0,1,0" );
    
    // Set SMS Text Mode Parameters
    at_cmd_single( "AT+CSMP=17,167,0,16" );

     // Power on GPS
    at_cmd_single( "AT+CGPSPWR=1" );

     // Reset and restart GPS
    at_cmd_single( "AT+CGPSRST=1" );
    
    LOG( "\r\n**********************************************************\r\n" );
    LOG( "Finished GSM / GNSS 2 Initialization." );
    LOG( "\r\n**********************************************************\r\n" );
}

/*
 * Composing Command and SMS content
 */
void send_SMS( void )
{
    char cmd_content[ 30 ] = { 0 };              // Send SMS command
    char reply_content[ 256 ] = { 0 };           // SMS content
    
    LOG( "\r\n**********************************************************\r\n" );
    LOG( "Sending SMS Message to Receiver..." );
    LOG( "\r\n**********************************************************\r\n" );

    strcat( cmd_content, "AT+CMGS=" );
    strcat( cmd_content, receiver_ID );
    strcat( reply_content, 
    "http://www.mikroe.com/\r\nThis example saves event log on microSD formed of messages and calls." );
    at_cmd_double( cmd_content, reply_content );   
}

/*
* Checking available memory, file creating, reading and writing to SD
*/
void micro_SD( void ) 
{   
    LOG( "\r\n**********************************************************\r\n" );
    LOG( "micro SD Initialization." );
    LOG( "\r\n**********************************************************\r\n" );
    
    at_cmd_single( "AT+FSDRIVE=1" );      // 1 - SD card drive , 0 - Local drive
      
    // Creating file mikroe.txt
    at_cmd_single( "AT+FSCREATE=D:\\mikroelektronika.txt" );

    // Creating file event_log.txt
    at_cmd_single( "AT+FSCREATE=D:\\event_log.txt" );  

    // Writing string into event_log file 
    at_cmd_single( "AT+FSWRITE=D:\\event_log.txt,0,11,1" );
    at_cmd_single( "SEQUENCE:\r\n" );

    Delay_ms( 1000 );
 
    // Writing string into mikroelektronika file 
    at_cmd_single( "AT+FSWRITE=D:\\mikroelektronika.txt,0,44,1" );
    at_cmd_single( "Mikroelektronika GSM/GNSS 2 click example\r\n" );

    Delay_ms( 1000 );
    
    // Reading file data of mikroe.txt
    at_cmd_single( "AT+FSREAD=D:\\mikroelektronika.txt,1,50,0" );
    
    // Checking the file size of mikroe.txt
    at_cmd_single( "AT+FSFLSIZE=D:\\mikroelektronika.txt" );
    
    at_cmd_single( "AT+FSMEM" );          // Get the Size of Available Memory
}

/*
 * Default response handler
 */
void rsp_handler( char *rsp )
{
    char tmp[ 25 ] = { 0 };
    
    LOG( "Response : \r\n" );
    LOG( rsp );
    LOG( "\r\n" );
    
    // Performing reset procedure in case of power down    
    if ( !strncmp( rsp, "\r\nNORMAL POWER DOWN", 19 ) )
    {
        GSM_GNSS_2_RST = 0;
        Delay_ms( 1500 );
        GSM_GNSS_2_RST = 1;
        LOG( "\r\n**********************************************************\r\n" );
        LOG( "Reset Done.\r\n" );
        LOG( "\r\n**********************************************************\r\n" );
    }
    

    if ( !strncmp( rsp, "\r\nRDY", 5 ) )
    {
        ready_f = true;
    }
    
    // Perform action after hang up
    if ( !strncmp( rsp, "\r\nNO CARRIER", 12 ) )
    {
        isCall = true;
        action_f = true;
    }
}


void sms_action_handler( char* rsp )
{ 
    isMessage = true;  
    action_f = true;
}

void at_cb_send_init_message ( char* rsp )
{
   msg_sent = true;
}

void at_cb_microsd_end ( char* rsp )
{
    LOG( rsp );
    sd_f = true;
}

void update_log ( void )
{
    char log_type[ 8 ] = { 0 };
      
    if ( isMessage )
    {
      strcpy( log_type, " msg  " );
      msg_counter++;
      isMessage = false;
    }
    if ( isCall )
    {
      strcpy( log_type, " call ");
      call_counter++;
      isCall = false;
    }

    LOG( "\r\nNumber of received messages is: ");
    ByteToStr( msg_counter, txt_msg );
    LOG( txt_msg );
    LOG( "\r\n" );
    LOG( "\r\nNumber of missed calls is: ");
    ByteToStr( call_counter, txt_call );
    LOG( txt_call );
    LOG( "\r\n" );

    // Save log to microSD card                   
    at_cmd_single( "AT+FSWRITE=D:\\event_log.txt,1,6,1" );
    at_cmd_single( log_type );  
    Delay_ms ( 1000 );

    LOG( "\r\n**********************************************************\r\n" );
    LOG( "GPS Location:\r\n" );
    LOG( "\r\n**********************************************************\r\n" );
   
    // Read GPS info
    at_cmd_single( "AT+CGPSINF=0" );
}

void main( void )
{
    system_init();
    at_init( rsp_handler, UART2_Write, buffer, sizeof( buffer ) );
    at_cmd_save( "+CMS ERROR", 1000, NULL, NULL, NULL, at_cb_send_init_message );
    at_cmd_save( "+FSMEM", 3000, NULL, NULL, NULL, at_cb_microsd_end );
    at_cmd_save( "+CMT", 3000, NULL, NULL, NULL, sms_action_handler );
    at_cmd_save( "+CMGS", 1000, NULL, NULL, NULL, at_cb_send_init_message );

    gsm_gnss_2_power_on();

    gsm_gnss_2_init();
    
    send_SMS();                               // Sending initial message
    
    while( !msg_sent ) at_process();          // Waiting for SMS sending status
    
    LOG( "\r\n**********************************************************\r\n" );
    LOG( "Finished Sending SMS Message.");
    LOG( "\r\n**********************************************************\r\n" );

    micro_SD();                               // Creating files on microSD
    
    while( !sd_f ) at_process();              // Waiting for SD card operations
    
    LOG( "\r\n**********************************************************\r\n" );
    LOG( "Finished SD Initialization.");
    LOG( "\r\n**********************************************************\r\n" );
    Delay_ms( 1000 );
    
    LOG( "\r\n**********************************************************\r\n" );
    LOG( "Waiting for the Message or Call in an Infinite Loop");
    LOG( "\r\n**********************************************************\r\n" );

    ////////////////////////////////////////////////////////////////////////////
    while( true )
    {
        at_process();

        if ( action_f )                       // Message received or call ended
        {
            update_log();
            action_f = false;
        }
    }
    ////////////////////////////////////////////////////////////////////////////
}

/*
 * Timer Interrupt
 */
void TICK_ISR() iv IVT_TIMERS_IRQ
{
    at_tick();
    if (TIMER_INT_A_bit){
    TIMER_INT = (TIMER_INT & 0xAA) | (1 << 0);
  }
}

/*
 * UART Interrupt
 */
void UART_RX_ISR() iv IVT_UART2_IRQ ics ICS_AUTO
{
    if( UART2_LSR & ( 1 << DATA_RDY ) )
    {
        char tmp_data = UART2_RBR;
        at_rx( tmp_data );
    }
}
/************************************************************ END OF FILE *****/